@@ -21,7 +21,7 @@ parameters or the exploration factor used to select actions.
 
 import functools
 import inspect
-from typing import Callable, Dict, Union, Optional, Iterable, Sequence
+from typing import Callable, Dict, Union, NamedTuple, Optional, Iterable, Sequence
 
 from absl import logging
 import chex
@@ -256,7 +256,7 @@ def piecewise_interpolate_schedule(
     init_value: An initial value `init_v`.
     boundaries_and_scales: A map from boundaries `b_i` to non-negative scaling
       factors `f_i`. At boundary step `b_i`, the schedule returns `init_v`
-      scaled by the product of all factors `f_j` such that `b_j` < `b_i`. The
+      scaled by the product of all factors `f_j` such that `b_j` <= `b_i`. The
       values in between each boundary will be interpolated as per `type`.
 
   Returns:
@@ -439,10 +439,11 @@ def warmup_exponential_decay_schedule(
     init_value: Initial value for the scalar to be annealed.
     peak_value: Peak value for scalar to be annealed at end of warmup.
     warmup_steps: Positive integer, the length of the linear warmup.
-    transition_steps: must be positive. See the decay computation above.
+    transition_steps: must be positive. See `exponential_decay` for more
+      details.
     decay_rate: must not be zero. The decay rate.
     transition_begin: must be positive. After how many steps to start annealing
-      (before this many steps the scalar value is held fixed at `init_value`).
+      (before this many steps the scalar value is held fixed at `peak_value`).
     staircase: if `True`, decay the values at discrete intervals.
     end_value: the value at which the exponential decay stops. When
       `decay_rate` < 1, `end_value` is treated as a lower bound, otherwise as
@@ -497,7 +498,7 @@ def _convert_floats(x, dtype):
   return x
 
 
-class InjectHyperparamsState(base.OptState):
+class InjectHyperparamsState(NamedTuple):
   """Maintains inner transform state, hyperparameters, and step count."""
   count: jnp.ndarray  # shape=(), dtype=jnp.int32
   hyperparams: Dict[str, chex.Numeric]
