@@ -30,7 +30,7 @@ from optax._src import wrappers
 # pylint:disable=no-value-for-parameter
 
 
-class TraceState(base.OptState):
+class TraceState(NamedTuple):
   """Holds an aggregation of past updates."""
   trace: base.Params
 
@@ -49,8 +49,8 @@ def trace(
   Args:
     decay: the decay rate for the trace of past updates.
     nesterov: whether to use Nesterov momentum.
-    accumulator_dtype: optional `dtype` to used for the accumulator; if `None`
-      then the `dtype` is inferred from `params` and `updates`.
+    accumulator_dtype: optional `dtype` to be used for the accumulator; if
+      `None` then the `dtype` is inferred from `params` and `updates`.
 
   Returns:
     An (init_fn, update_fn) tuple.
@@ -87,7 +87,7 @@ def _bias_correction(moment, decay, count):
   return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
 
 
-class EmaState(base.OptState):
+class EmaState(NamedTuple):
   """Holds an exponential moving average of past updates."""
   count: chex.Array  # shape=(), dtype=jnp.int32.
   ema: base.Params
@@ -134,7 +134,7 @@ def ema(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRssState(base.OptState):
+class ScaleByRssState(NamedTuple):
   """State holding the sum of gradient squares to date."""
   sum_of_squares: base.Updates
 
@@ -175,7 +175,7 @@ def scale_by_rss(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRmsState(base.OptState):
+class ScaleByRmsState(NamedTuple):
   """State for exponential root mean-squared (RMS)-normalized updates."""
   nu: base.Updates
 
@@ -214,7 +214,7 @@ def scale_by_rms(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByRStdDevState(base.OptState):
+class ScaleByRStdDevState(NamedTuple):
   """State for centered exponential moving average of squares of updates."""
   mu: base.Updates
   nu: base.Updates
@@ -257,7 +257,7 @@ def scale_by_stddev(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByAdamState(base.OptState):
+class ScaleByAdamState(NamedTuple):
   """State for the Adam algorithm."""
   count: chex.Array  # shape=(), dtype=jnp.int32.
   mu: base.Updates
@@ -268,7 +268,8 @@ def scale_by_adam(
     b1: float = 0.9,
     b2: float = 0.999,
     eps: float = 1e-8,
-    eps_root: float = 0.0
+    eps_root: float = 0.0,
+    mu_dtype: Optional[Any] = None,
 ) -> base.GradientTransformation:
   """Rescale updates according to the Adam algorithm.
 
@@ -281,13 +282,18 @@ def scale_by_adam(
     eps: term added to the denominator to improve numerical stability.
     eps_root: term added to the denominator inside the square-root to improve
       numerical stability when backpropagating gradients through the rescaling.
+    mu_dtype: optional `dtype` to be used for the first order accumulator; if
+      `None` then the `dtype is inferred from `params` and `updates`.
 
   Returns:
     An (init_fn, update_fn) tuple.
   """
 
+  mu_dtype = utils.canonicalize_dtype(mu_dtype)
+
   def init_fn(params):
-    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
+    mu = jax.tree_map(  # First moment
+        lambda t: jnp.zeros_like(t, dtype=mu_dtype), params)
     nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
     return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
 
@@ -296,7 +302,7 @@ def scale_by_adam(
     mu = _update_moment(updates, state.mu, b1, 1)
     nu = _update_moment(updates, state.nu, b2, 2)
     count_inc = numerics.safe_int32_increment(state.count)
-    mu_hat = _bias_correction(mu, b1, count_inc)
+    mu_hat = utils.cast_tree(_bias_correction(mu, b1, count_inc), mu_dtype)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
@@ -389,7 +395,7 @@ def scale_by_param_block_rms(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByBeliefState(base.OptState):
+class ScaleByBeliefState(NamedTuple):
   """State for the rescaling by AdaBelief algorithm."""
   count: chex.Array  # shape=(), dtype=jnp.int32.
   mu: base.Updates
@@ -399,7 +405,7 @@ class ScaleByBeliefState(base.OptState):
 def scale_by_belief(
     b1: float = 0.9,
     b2: float = 0.999,
-    eps: float = 0.,
+    eps: float = 1e-16,
     eps_root: float = 1e-16
 ) -> base.GradientTransformation:
   """Rescale updates according to the AdaBelief algorithm.
@@ -411,8 +417,9 @@ def scale_by_belief(
     b1: decay rate for the exponentially weighted average of grads.
     b2: decay rate for the exponentially weighted average of variance of grads.
     eps: term added to the denominator to improve numerical stability.
-    eps_root: term added to the denominator inside the square-root to improve
-      numerical stability when backpropagating gradients through the rescaling.
+    eps_root: term added to the second moment of the prediction error to
+      improve numerical stability. If backpropagating gradients through the
+      gradient transformation (e.g. for meta-learning), this must be non-zero.
 
   Returns:
     An (init_fn, update_fn) tuple.
@@ -428,11 +435,12 @@ def scale_by_belief(
     mu = _update_moment(updates, state.mu, b1, 1)
     prediction_error = jax.tree_multimap(lambda g, m: g-m, updates, state.mu)
     nu = _update_moment(prediction_error, state.nu, b2, 2)
+    nu = jax.tree_map(lambda v: v + eps_root, nu)
     count_inc = numerics.safe_int32_increment(state.count)
     mu_hat = _bias_correction(mu, b1, count_inc)
     nu_hat = _bias_correction(nu, b2, count_inc)
     updates = jax.tree_multimap(
-        lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
+        lambda m, v: m / (jnp.sqrt(v) + eps), mu_hat, nu_hat)
     return updates, ScaleByBeliefState(count=count_inc, mu=mu, nu=nu)
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -579,7 +587,7 @@ def add_decayed_weights(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByScheduleState(base.OptState):
+class ScaleByScheduleState(NamedTuple):
   """Maintains count for scale scheduling."""
   count: chex.Array  # shape=(), dtype=jnp.int32
 
@@ -611,7 +619,7 @@ def scale_by_schedule(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleByFromageState(base.OptState):
+class ScaleByFromageState(NamedTuple):
   """Maintains count for step-size scheduling."""
   count: chex.Array  # shape=(), dtype=jnp.int32
 
@@ -667,7 +675,7 @@ def scale_by_trust_ratio(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class AddNoiseState(base.OptState):
+class AddNoiseState(NamedTuple):
   """State for adding gradient noise. Contains a count for annealing."""
   count: chex.Array
   rng_key: chex.PRNGKey
@@ -714,7 +722,7 @@ def add_noise(
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ApplyEvery(base.OptState):
+class ApplyEvery(NamedTuple):
   """Contains a counter and a gradient accumulator."""
   count: chex.Array
   grad_acc: base.Updates
@@ -787,7 +795,7 @@ def centralize() -> base.GradientTransformation:
   return base.GradientTransformation(init_fn, update_fn)
 
 
-class ScaleBySM3State(base.OptState):
+class ScaleBySM3State(NamedTuple):
   """State for the SM3 algorithm."""
   mu: base.Updates
   nu: base.Updates
