@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
+
+
 """Tests for `transform.py`."""
 
 from absl.testing import absltest
@@ -21,6 +22,7 @@ from absl.testing import parameterized
 import chex
 import jax
 import jax.numpy as jnp
+import numpy as np
 
 from optax._src import alias
 from optax._src import combine
@@ -44,6 +46,8 @@ class TransformTest(parameterized.TestCase):
       ('rmsprop', transform.scale_by_rms),
       ('stddev', transform.scale_by_stddev),
       ('trust_ratio', transform.scale_by_trust_ratio),
+      ('param_block_norm', transform.scale_by_param_block_norm),
+      ('param_block_rms', transform.scale_by_param_block_rms),
   ])
   def test_scalers(self, scaler_constr):
     params = self.init_params
@@ -60,6 +64,75 @@ class TransformTest(parameterized.TestCase):
     jax.tree_multimap(lambda *args: chex.assert_equal_shape(args), params,
                       updates)
 
+  @chex.all_variants()
+  def test_add_decayed_weights(self):
+    # Define a transform that add decayed weights.
+    # We can define a mask either as a pytree, or as a function that
+    # returns the pytree. Below we define the pytree directly.
+    mask = (True, dict(a=True, b=False))
+    tx = transform.add_decayed_weights(0.1, mask=mask)
+    # Define input updates and weights.
+    updates = (
+        jnp.zeros((2,), dtype=jnp.float32),
+        dict(
+            a=jnp.zeros((2,), dtype=jnp.float32),
+            b=jnp.zeros((2,), dtype=jnp.float32),))
+    weights = (
+        jnp.ones((2,), dtype=jnp.float32),
+        dict(
+            a=jnp.ones((2,), dtype=jnp.float32),
+            b=jnp.ones((2,), dtype=jnp.float32),))
+    # This mask means that we will add decayed weights to the first two
+    # terms in the input updates, but not to the last element.
+    expected_tx_updates = (
+        0.1*jnp.ones((2,), dtype=jnp.float32),
+        dict(
+            a=0.1*jnp.ones((2,), dtype=jnp.float32),
+            b=jnp.zeros((2,), dtype=jnp.float32),))
+    # Apply transform
+    state = tx.init(weights)
+    transform_fn = self.variant(tx.update)
+    new_updates, _ = transform_fn(updates, state, weights)
+    # Assert output as expected.
+    chex.assert_tree_all_close(new_updates, expected_tx_updates)
+
+  @chex.all_variants()
+  def test_ema(self):
+    values = jnp.array([5.0, 7.0])
+    decay = 0.9
+    d = decay
+
+    ema = transform.ema(decay=decay, debias=False)
+    state = ema.init(values[0])  # init to zeroes
+
+    transform_fn = self.variant(ema.update)
+    mean, state = transform_fn(values[0], state)
+    np.testing.assert_allclose(mean, (1-d) * values[0], atol=1e-4)
+
+    mean, state = transform_fn(values[1], state)
+    np.testing.assert_allclose(
+        mean,
+        (1 - d) * (values[1] + d * values[0]), atol=1e-2)
+
+  @chex.all_variants()
+  def test_ema_debias(self):
+    values = jnp.array([5.0, 7.0])
+    decay = 0.9
+    d = decay
+
+    ema = transform.ema(decay=decay)
+    state = ema.init(values[0])
+
+    transform_fn = self.variant(ema.update)
+    mean, state = transform_fn(values[0], state)
+    np.testing.assert_allclose(mean, values[0], atol=1e-4)
+
+    mean, state = transform_fn(values[1], state)
+    np.testing.assert_allclose(
+        mean,
+        ((1 - d) * values[1] + d * values[0]) / (1 - d**2),
+        atol=1e-2)
+
   @chex.all_variants()
   def test_apply_every(self):
     # The frequency of the application of sgd
@@ -115,29 +188,6 @@ class TransformTest(parameterized.TestCase):
       # Check the rescaled updates match.
       chex.assert_tree_all_close(scaled_updates, manual_updates)
 
-  def test_clip(self):
-    updates = self.per_step_updates
-    # For a sufficiently high delta the update should not be changed.
-    clipper = transform.clip(1e6)
-    clipped_updates, _ = clipper.update(updates, None)
-    chex.assert_tree_all_close(clipped_updates, clipped_updates)
-    # Clipping at delta=1 should make all updates exactly 1.
-    clipper = transform.clip(1.)
-    clipped_updates, _ = clipper.update(updates, None)
-    chex.assert_tree_all_close(
-        clipped_updates, jax.tree_map(jnp.ones_like, updates))
-
-  def test_clip_by_global_norm(self):
-    updates = self.per_step_updates
-    for i in range(1, STEPS + 1):
-      clipper = transform.clip_by_global_norm(1. / i)
-      updates, _ = clipper.update(updates, None)
-      # Check that the clipper actually works and global norm is <= max_norm
-      self.assertAlmostEqual(transform.global_norm(updates), 1. / i, places=6)
-      updates_step, _ = clipper.update(self.per_step_updates, None)
-      # Check that continuously clipping won't cause numerical issues.
-      chex.assert_tree_all_close(updates, updates_step, atol=1e-7, rtol=1e-7)
-
   @parameterized.named_parameters([
       ('1d', [1.0, 2.0], [1.0, 2.0]),
       ('2d', [[1.0, 2.0], [3.0, 4.0]], [[-0.5, 0.5], [-0.5, 0.5]]),
@@ -152,85 +202,6 @@ class TransformTest(parameterized.TestCase):
     centralized_inputs, _ = centralizer.update(inputs, None)
     chex.assert_tree_all_close(centralized_inputs, outputs)
 
-  def test_keep_params_nonnegative(self):
-    grads = (jnp.array([500., -500., 0.]),
-             jnp.array([500., -500., 0.]),
-             jnp.array([500., -500., 0.]))
-
-    params = (jnp.array([-1., -1., -1.]),
-              jnp.array([1., 1., 1.]),
-              jnp.array([0., 0., 0.]))
-
-    # vanilla sgd
-    opt = combine.chain(
-        transform.trace(decay=0, nesterov=False), transform.scale(-LR))
-    opt_state = opt.init(params)
-
-    updates, _ = opt.update(grads, opt_state, params)
-    new_params = update.apply_updates(params, updates)
-
-    chex.assert_tree_all_close(new_params, (jnp.array([-6., 4., -1.]),
-                                            jnp.array([-4., 6., 1.]),
-                                            jnp.array([-5., 5., 0.])))
-
-    # sgd with keeping parameters non-negative
-    opt = combine.chain(
-        transform.trace(decay=0, nesterov=False), transform.scale(-LR),
-        transform.keep_params_nonnegative())
-    opt_state = opt.init(params)
-
-    updates, _ = opt.update(grads, opt_state, params)
-    new_params = update.apply_updates(params, updates)
-
-    chex.assert_tree_all_close(new_params, (jnp.array([0., 4., 0.]),
-                                            jnp.array([0., 6., 1.]),
-                                            jnp.array([0., 5., 0.])))
-
-  @chex.all_variants
-  def test_zero_nans(self):
-    params = (jnp.zeros([3]), jnp.zeros([3]), jnp.zeros([3]))
-
-    opt = transform.zero_nans()
-    opt_state = self.variant(opt.init)(params)
-    update_fn = self.variant(opt.update)
-
-    equality_comp = lambda a, b: bool(jnp.all(jnp.equal(a, b)))
-    chex.assert_tree_all_equal_comparator(equality_comp, opt_state,
-                                          (jnp.array(False),) * 3)
-
-    # Check an upate with nans
-    grads_with_nans = (jnp.ones([3]),
-                       jnp.array([1., float('nan'), float('nan')]),
-                       jnp.array([float('nan'), 1., 1.]))
-    updates, opt_state = update_fn(grads_with_nans, opt_state)
-    chex.assert_tree_all_equal_comparator(
-        equality_comp, opt_state,
-        (jnp.array(False), jnp.array(True), jnp.array(True)))
-    chex.assert_tree_all_equal_comparator(
-        equality_comp, updates,
-        (jnp.ones([3]), jnp.array([1., 0., 0.]), jnp.array([0., 1., 1.])))
-
-    # Check an upate with nans and infs
-    grads_with_nans_infs = (jnp.ones([3]),
-                            jnp.array([1., float('nan'), float('nan')]),
-                            jnp.array([float('inf'), 1., 1.]))
-    updates, opt_state = update_fn(grads_with_nans_infs, opt_state)
-    chex.assert_tree_all_equal_comparator(
-        equality_comp, opt_state,
-        (jnp.array(False), jnp.array(True), jnp.array(False)))
-    chex.assert_tree_all_equal_comparator(
-        equality_comp, updates,
-        (jnp.ones([3]), jnp.array([1., 0., 0.]),
-         jnp.array([float('inf'), 1., 1.])))
-
-    # Check an upate with only good values
-    grads = (jnp.ones([3]), jnp.ones([3]), jnp.ones([3]))
-    updates, opt_state = update_fn(grads, opt_state)
-    chex.assert_tree_all_equal_comparator(
-        equality_comp, opt_state,
-        (jnp.array(False), jnp.array(False), jnp.array(False)))
-    chex.assert_tree_all_equal_comparator(equality_comp, updates, grads)
-
 
 if __name__ == '__main__':
   absltest.main()
